<?php
namespace Teknisa\GOP\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;

class Gop10001SimulacaoValoresController extends \Teknisa\GER\Controllers\TeknisaCrud {
    
    protected $parameterBag;
    protected $util;
    protected $environment;
    protected $dataSourceName;
    protected $parameterBagColums;
    protected $dataSourceManager;
    protected $service;
    
    public function __construct($crudService, $parameterBag, $service, $util, $environment, $dataSourceManager){
        $this->crudService          = $crudService;
        $this->service              = $service;
        $this->util                 = $util;
        $this->environment          = $environment;
        $this->dataSourceManager    = $dataSourceManager;
    }
   
    public function filterSimulation(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        try {
            if(isset($row['NRLISTA']) && $row['NRLISTA'] != '') {
                $this->util->deleteLista($row['NRLISTA']);
            }
            $nrLista = $this->util->insertLista($row['CDFILIAL']);
            $conditions = array(
                array(
                    "columnName"=>"P_NRLISTA", 
                    "operator"=>"=", 
                    "value"=>$nrLista
                ),
                array(
                    "columnName"=>"P_IDPERIODOORCA", 
                    "operator"=>"=", 
                    "value"=>$row["P_IDPERIODOORCA"]
                ),
                array(
                    "columnName"=>"P_NRPERIODOORCA", 
                    "operator"=>"=", 
                    "value"=>$row["P_NRPERIODOORCA"]
                ),
                array(
                    "columnName"=>"P_NRANOORCAGOP", 
                    "operator"=>"=", 
                    "value"=>$row["P_NRANOORCAGOP"]
                ),
                array(
                    "columnName"=>"P_NRORG", 
                    "operator"=>"=", 
                    "value"=>$this->environment->getNrOrg()
                )
            );
            $result = $this->crudService->getFilteredDataSet('gop10001_simulacaoValores', $conditions);
            $dados = $result->getRows();
            $dataSet = array(
                "data"=>$dados,
                "nrLista"=>$nrLista
            );
            $response->addDataSet(new DataSet('success', $dataSet));
        }catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500)); 
        }
    }
    
    public function verifyParamVrFaturamento(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        try{
            if(!is_string($row["CDFILIAL"]) && count($row["CDFILIAL"]) > 1){
                $row["CDFILIAL"] = "P";
            }
            
            $paramVrFaturamento = $this->service->verifyParamVrFaturamento($row);
            $dataSet = array (
                'VRFATURAMENTO' => $paramVrFaturamento
            );
            $response->addDataSet(new DataSet('result', $dataSet));
            
        }catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function saveCalculation(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        try{
            $this->service->beginTransaction();
            if(array_key_exists('BACKUP', $row)) {
                $arrayData = array();
                $arrayKeysDataLoaded = $this->getLoadedData($row['BACKUP'], $row['DATASIMULATION'], $arrayData);
                $list = $this->util->insertLista($arrayKeysDataLoaded);
                $this->service->getDataNotExistsBkpFatCons($list, $row['PARAMSSIMULATION'], $arrayData, $row['NRLISTA']);
                $this->service->getDataNotExistsBkpMaoObra($list, $row['PARAMSSIMULATION'], $arrayData, $row['NRLISTA']);
                $this->service->getDataNotExistsBkpDetail($list, $row['PARAMSSIMULATION'], $arrayData, $row['NRLISTA']);
                $this->rateioDetalhamento($arrayData, $row['DATASIMULATION'], $row['FILIAIS']);
            
                $this->updateCalc($arrayData, $row['PARAMSSIMULATION']);
                $this->util->deleteLista($list);
            }
            $this->service->commit();
        }catch(\Exception $e) {
            $this->service->rollback();
            $response->setError(new Error ($e->getMessage(), 500));
            $this->util->deleteLista($list); 
        }
    }
    
    private function getLoadedData($backup, $dataSimulation, &$arrayData) {
        $arrayKeysDataLoaded = array();
        $stringConcat = '';
        foreach($backup as $keyUnit => $unit) { 
            foreach($unit as $keyIdTipoOrca => $idTipoOrca) { 
                if($keyIdTipoOrca != 'VRFATURAMENTO') { 
                    if(count($idTipoOrca)) {
                        foreach($idTipoOrca as $keyDetailUnit => $detailUnit) {
                            if((array_key_exists('MESES', $detailUnit)) || (!array_key_exists('MESES', $detailUnit) && !isset($dataSimulation[$keyIdTipoOrca]['CONFIG']['RATEIO'])) && !isset($dataSimulation[$keyIdTipoOrca]['CONFIG']['RATEIO_ZERADO'])) {
                                $stringConcat = $keyUnit.$keyIdTipoOrca.substr($keyDetailUnit, 1);
                                array_push($arrayKeysDataLoaded, $stringConcat);
                            }
                            
                            if(array_key_exists('MESES', $detailUnit)) {
                                foreach($detailUnit['MESES'] as $key => $month) {
                                    array_push($arrayData, $month); 
                                }
                            }
                        }
                    }
                }
            }
        }
        return $arrayKeysDataLoaded;
    }
    
    private function rateioDetalhamento(&$arrayData, $dataSimulation, $filiais) {
        foreach($arrayData as &$row) {
            $typeBudget = $row['IDTIPOORCAOP'];
            if($typeBudget != 'IMPOSTOARECUPERAR') {
                $config = $dataSimulation[$typeBudget]['CONFIG'];
                $cdFilial = $row['CDFILIAL'];
                $param = $filiais[$cdFilial];
                   
                $percent = array_key_exists('ACCUMULATOR_RATEIO', $config) && $config['ACCUMULATOR_RATEIO'] == 1 ? $config['RATEIO'] :
                    (array_key_exists('ACCUMULATOR_RATEIO', $config) ? $config['ACCUMULATOR_RATEIO'] : null);
                
                if(isset($config['RATEIO_ZERADO_MONTH']) && !isset($config['RATEIO']) && isset($row['CDSERVFILI']) ) {
                    if(floatval($row['VRPRECOBASE']) == 0) {
                        $row['VRPRECOBASE'] = 1;
                    }
                    
                    if(floatval($row['CUSTPERCAPITA']) == 0) {
                        $row['CUSTPERCAPITA'] = 1;
                    }
                    
                    $row['QTORCAMENTO'] = $config['RATEIO_ZERADO_MONTH'];
                    
                    if($row['IDTIPOORCAOP'] === 'FATURAMENTO') {
                        $row['VRORCAMENTO'] = $config['RATEIO_ZERADO_MONTH'] * $row['VRPRECOBASE'];
                    }
                
                    if($row['IDTIPOORCAOP'] === 'CONSUMO') {
                        $row['VRORCAMENTO'] = $config['RATEIO_ZERADO_MONTH'] * $row['CUSTPERCAPITA'];
                    }
                }else if(isset($config['RATEIO']) && isset($row["CDSERVFILI"])) {
                    if(isset($row['VRPRECOBASE']) && $row['VRPRECOBASE'] == 0) {
                        $row['VRPRECOBASE'] = 1;
                    }
                    
                    if(isset($row['CUSTPERCAPITA']) && floatval($row['CUSTPERCAPITA']) == 0) {
                        $row['CUSTPERCAPITA'] = 1;
                    }
                    
                    if(!isset($config['RATEIO_ZERADO_MONTH'])) {
                        $row['QTORCAMENTO'] = floatval($row['QTORCAMENTO']) * $percent;
                    }else{
                        $row['QTORCAMENTO'] = $config['RATEIO_ZERADO_MONTH'] * $percent;
                    }
                    
                    if($row['IDTIPOORCAOP'] === 'FATURAMENTO') {
                        $row['VRORCAMENTO'] = $row["QTORCAMENTO"] * $row['VRPRECOBASE'];
                    }
                    
                    if($row['IDTIPOORCAOP'] === 'CONSUMO') {
                        $row['VRORCAMENTO'] = $row['QTORCAMENTO'] * $row['CUSTPERCAPITA'];
                    }
                }
                
                if(isset($config['RATEIO_ZERADO_MONTH']) && !isset($config['RATEIO']) && isset($row['CDSERVFILI']) === null) {
                    $row['VRORCAMENTO'] = $config['RATEIO_ZERADO_MONTH'];
                }else if(isset($config['RATEIO']) && isset($row['CDSERVFILI']) === null) {
                    if(!isset($config['RATEIO_ZERADO_MONTH'])) {
                        $row['VRORCAMENTO'] = $row['VRORCAMENTO'] * $percent;
                    }else{
                        $row['VRORCAMENTO'] = $config['RATEIO_ZERADO_MONTH'] * $percent;
                    }
                }
                $row['VRPERCFAT'] = floatval($dataSimulation[$param]['VRORCAMENTO']) == 0 ? 0 : (floatval($row['VRORCAMENTO']) * 100) / floatval($dataSimulation[$param]['VRORCAMENTO']);
            }
        }
    }
    
    private function updateCalc(&$arrayData, $paramsSimulation) {
        foreach($arrayData as &$row) {
            if(isset($row['NRITDETALORCAMESGOP'])) {
                $this->service->updateCalcItDetalOrcaMesGop($row);
            }
            $this->service->updateCalcItOrcaMesGopDetail($row, $paramsSimulation);
            $this->service->updateCalcItOrcaMesGop($row);
        }
    }
}