<?php
namespace Teknisa\GOP\Services;

 use Zeedhi\Framework\DataSource\DataSet;
 use Zeedhi\Framework\DTO;
 
 class Gop10001SimulacaoValoresService {
    protected $connection;
    
    public function __construct($connection, $environment){
        $this->connection = $connection;
        $this->environment = $environment;
    }
    
    public function beginTransaction() {
        $this->connection->beginTransaction();
    }
    
    public function commit() {
        $this->connection->commit();
    }
    
    public function rollback() {
        $this->connection->rollback();
    }
    
    public function verifyParamVrFaturamento($row) {
        $select = "SELECT F.*, (SELECT
                        CASE NVL(MAX(IDCALCPERCENTFAT), 'FATURAMENTOLIQUIDO')
                            WHEN 'FATURAMENTO_BRUTO' THEN
                                'FATURAMENTO'
                            ELSE
                                'FATURAMENTOLIQUIDO'
                        END IDCALCPERCENTFAT
                    FROM
                        PARFILIALROP P
                    WHERE
                        P.CDFILIAL = TIPOPARAM_DRO(CDFILIAL)) VRFATURAMENTO FROM FILIAL F";
        
        $params = array (
            "P_CDFILIAL" => $row["CDFILIAL"]
        );
        
        try{
            $std = $this->connection->prepare($select);
            $std->execute($params);
            $result = $std->fetchAll();
            return $result[0]['IDCALCPERCENTFAT'];
        }catch (Exception $e){
            throw $e;
        }  
    } 
    
    public function getDataNotExistsBkpFatCons($list, $paramsSimulation, &$arrayData, $listUnit) {
        $select = "SELECT
                        O.CDFILIAL,
                        D.CDSERVFILI,
                        D.CDPRODUTO,
                        D.NRITDETALORCAMESGOP,
                        NVL(D.QTORCAMENTO, 0) QTORCAMENTO,
                        NVL(D.QTORCAMENTO, 0) QTESTIMADA,
                        NVL(D.VRPRECOBASE, 0) VRPRECOBASE,
                        NVL(D.VRORCAMENTO, 0) VRORCAMENTO,
                        M.DTORCAMENTO NRMES,
                        M.NRITORCAMESGOP,
                        M.DTORCAMENTO NRMES,
                        DECODE(D.QTORCAMENTO, 0, 0,(NVL(D.VRORCAMENTO, 0) / D.QTORCAMENTO)) CUSTPERCAPITA,
                        I.IDTIPOORCAOP
                    FROM
                        ORCAMENTOGOP        O
                        INNER JOIN ITEMORCAGOP         I ON I.NRORCAMENTOGOP = O.NRORCAMENTOGOP
                        INNER JOIN ITORCAMESGOP        M ON M.NRITEMORCAOP = I.NRITEMORCAOP
                        INNER JOIN ITDETALORCAMESGOP   D ON D.NRITORCAMESGOP = M.NRITORCAMESGOP
                        LEFT JOIN LISTA               L ON L.NRLISTA = :P_NRLISTA
                                             AND L.CDLISTA = ( O.CDFILIAL
                                                               || I.IDTIPOORCAOP
                                                               || NVL(D.CDSERVFILI, D.CDPRODUTO) )
                        LEFT JOIN LISTA             J ON J.NRLISTA = :P_NRLISTAUNIT
                                                    AND J.CDLISTA = O.CDFILIAL                                        
                    WHERE
                        O.NRANOORCAGOP = :P_ANO
                        AND O.IDPERIODOORCA = :P_IDPERIODOORCA
                        AND O.NRPERIODOORCA = :P_NRPERIODOORCA
                        AND O.NRORG = :P_NRORG
                        AND I.IDTIPOORCAOP IN (
                            'FATURAMENTO',
                            'CONSUMO'
                        )
                        AND L.CDLISTA IS NULL
                        AND J.CDLISTA IS NOT NULL
                    ORDER BY
                        O.CDFILIAL,
                        M.DTORCAMENTO"; 
                    
        $params = array(
            ":P_NRLISTA" => $list,
            ":P_ANO" => $paramsSimulation["P_ANO"],
            ":P_IDPERIODOORCA" => $paramsSimulation["P_IDPERIODOORCA"],
            ":P_NRPERIODOORCA" => $paramsSimulation["P_NRPERIODOORCA"],
            ":P_NRORG"=> $this->environment->getNrOrg(),
            ":P_NRLISTAUNIT" => $listUnit
        );
        
        try{
            $std = $this->connection->prepare($select);
            $std->execute($params);
            $result = $std->fetchAll();
            
            $arrayData = array_merge($arrayData, $result);
        }catch(Exception $e){
            throw $e;
        }
    }
    
    public function getDataNotExistsBkpMaoObra($list, $paramsSimulation, &$arrayData, $listUnit) {
        $select = "SELECT
                        O.CDFILIAL,
                        I.CDORCAMENTOMO,
                        D.CDDESPADMFAT,
                        D.NRITDETALORCAMESGOP,
                        NVL(M.VRORCAMENTO, 0) VRORCAMENTO,
                        M.DTORCAMENTO NRMES,
                        I.IDTIPOORCAOP,
                        M.NRITORCAMESGOP
                    FROM
                        ORCAMENTOGOP        O
                        INNER JOIN ITEMORCAGOP         I ON I.NRORCAMENTOGOP = O.NRORCAMENTOGOP
                        INNER JOIN ITORCAMESGOP        M ON M.NRITEMORCAOP = I.NRITEMORCAOP
                        INNER JOIN ITDETALORCAMESGOP   D ON M.NRITORCAMESGOP = D.NRITORCAMESGOP
                        LEFT JOIN LISTA               L ON L.NRLISTA = :P_NRLISTA
                                                        AND L.CDLISTA = ( I.IDTIPOORCAOP || O.CDFILIAL || I.CDORCAMENTOMO || D.CDDESPADMFAT )
                    LEFT JOIN LISTA                 J ON J.NRLISTA = :P_NRLISTAUNIT
                                                    AND J.CDLISTA = O.CDFILIAL                                 
                    WHERE
                        O.NRANOORCAGOP = :P_ANO
                        AND O.IDPERIODOORCA = :P_IDPERIODOORCA
                            AND O.NRPERIODOORCA = :P_NRPERIODOORCA
                                AND O.NRORG = :P_NRORG
                                    AND I.IDTIPOORCAOP = 'MAOOBRA'
                    AND L.CDLISTA IS NULL
                    AND J.CDLISTA IS NOT NULL
                    ORDER BY
                        O.CDFILIAL,
                        M.DTORCAMENTO";
        
        $params = array (
            ":P_NRLISTA" => $list,
            ":P_ANO" => $paramsSimulation["P_ANO"],
            ":P_IDPERIODOORCA" => $paramsSimulation["P_IDPERIODOORCA"],
            ":P_NRPERIODOORCA" => $paramsSimulation["P_NRPERIODOORCA"],
            ":P_NRORG"=> $this->environment->getNrOrg(),
            ":P_NRLISTAUNIT" => $listUnit
        );

        try{
            $std = $this->connection->prepare($select);
            $std->execute($params);
            $result = $std->fetchAll();
    
            $arrayData = array_merge($arrayData, $result);
        }catch(Exception $e){
            throw $e;
        }
    }
    
    public function getDataNotExistsBkpDetail($list, $paramsSimulation, &$arrayData, $listUnit) {
        $select = "SELECT
                        O.CDFILIAL,
                        NVL(I.CDINVEST, NVL(I.CDIMPOSTO, I.CDDESPPREV)) CODIGO,
                        I.CDINVEST,
                        I.CDIMPOSTO,
                        I.CDDESPPREV,
                        M.VRPERCENTUAL,
                        NVL(M.VRORCAMENTO, 0) VRORCAMENTO,
                        M.DTORCAMENTO NRMES,
                        IDTIPOORCAOP,
                         M.NRITORCAMESGOP
                    FROM
                        ORCAMENTOGOP        O
                        INNER JOIN ITEMORCAGOP         I ON I.NRORCAMENTOGOP = O.NRORCAMENTOGOP
                        INNER JOIN ITORCAMESGOP        M ON M.NRITEMORCAOP = I.NRITEMORCAOP
                        LEFT JOIN LISTA               L ON L.NRLISTA = :P_NRLISTA
                                             AND L.CDLISTA = ( O.CDFILIAL
                                                               || I.IDTIPOORCAOP
                                                               || NVL(I.CDINVEST, NVL(I.CDIMPOSTO, I.CDDESPPREV)) )
                         LEFT JOIN LISTA          J ON J.NRLISTA = :P_NRLISTAUNIT
                                                    AND J.CDLISTA = O.CDFILIAL                                      
                    WHERE
                        O.NRANOORCAGOP = :P_ANO
                        AND O.IDPERIODOORCA = :P_IDPERIODOORCA
                        AND O.NRPERIODOORCA = :P_NRPERIODOORCA
                        AND O.NRORG = :P_NRORG
                        AND I.IDTIPOORCAOP IN (
                            'INVESTIMENTO',
                            'IMPOSTO',
                            'DESPMATRIZ',
                            'DESPDIVERSA',
                            'DESPENTRADA',
                            'IMPOSTOARECUPERAR',
                            'IMPOSTOSOBLUCRO',
                            'TAXAADM',
                            'INVESTIMENTO',
                            'DEPRECIACAO'
                        )
                        AND L.CDLISTA IS NULL
                        AND J.CDLISTA IS NOT NULL
                    ORDER BY
                        O.CDFILIAL,
                        M.DTORCAMENTO";
                        
        $params = array(
            ":P_NRLISTA" => $list,
            ":P_ANO" => $paramsSimulation["P_ANO"],
            ":P_IDPERIODOORCA" => $paramsSimulation["P_IDPERIODOORCA"],
            ":P_NRPERIODOORCA" => $paramsSimulation["P_NRPERIODOORCA"],
            ":P_NRORG"=> $this->environment->getNrOrg(),
            ":P_NRLISTAUNIT" => $listUnit
        );
        try {
            $std = $this->connection->prepare($select);
            $std->execute($params);
            $result = $std->fetchAll();
           
           $arrayData = array_merge($arrayData, $result);
        }catch(Exception $e){
            throw $e;
        }
    }
    
    public function updateCalcItDetalOrcaMesGop($row) {
        $sqlUpdate = "UPDATE ITDETALORCAMESGOP
                         SET
                             VRORCAMENTO = :P_VRORCAMENTO,
                             QTORCAMENTO = :P_QTORCAMENTO,
                             VRPRECOBASE = :P_VRPRECOBASE
                        WHERE
                             NRORG = :P_NRORG
                             AND NRITDETALORCAMESGOP = :P_NRITDETALORCAMESGOP";
                             
        $params = array(
            ":P_VRORCAMENTO" => floatval($row['VRORCAMENTO']),
            ":P_QTORCAMENTO" => isset($row['QTORCAMENTO']) ? floatval($row['QTORCAMENTO']) : 0,
            ":P_VRPRECOBASE" => isset($row['VRPRECOBASE']) ? floatval($row['VRPRECOBASE']) : 0,
            ":P_NRORG" => $this->environment->getNrOrg(),
            ":P_NRITDETALORCAMESGOP" => $row['NRITDETALORCAMESGOP']
        );
        
        try{
            $std = $this->connection->prepare($sqlUpdate);
            $std->execute($params);
        }catch(Exception $e) {
            throw $e;
        }
    }
    
    public function updateCalcItOrcaMesGopDetail($row, $paramsSimulation) {
        $sqlUpdate = "UPDATE 
                            ITORCAMESGOP
                        SET
                            VRORCAMENTO = (
                                SELECT
                                    SUM(D.VRORCAMENTO)
                                FROM
                                    ITDETALORCAMESGOP D
                                WHERE
                                    D.NRITORCAMESGOP = ITORCAMESGOP.NRITORCAMESGOP
                                
                            )WHERE NRITORCAMESGOP = :P_NRITORCAMESGOP";
        $params = array(
            ":P_NRITORCAMESGOP" => $row['NRITORCAMESGOP']
        );

        try{
            $std = $this->connection->prepare($sqlUpdate);
            $std->execute($params);
        }catch(Exception $e){
            throw $e;
        } 
    }
    
    public function updateCalcItOrcaMesGop($row) {
        $sqlUpdate = "UPDATE ITORCAMESGOP
                         SET
                             VRORCAMENTO = :P_VRORCAMENTO,
                             QTORCAMENTO = :P_QTORCAMENTO
                       WHERE
                             NRITORCAMESGOP = :P_NRITORCAMESGOP
                         AND 
                          NOT EXISTS (
                                SELECT 
                                    NULL  
                                FROM 
                                    ITDETALORCAMESGOP D
                                WHERE 
                                NRORG = :P_NRORG
                                AND D.NRITORCAMESGOP = ITORCAMESGOP.NRITORCAMESGOP 
                          )";
                     
        $params = array(
            ":P_VRORCAMENTO" => floatval($row['VRORCAMENTO']),
            ":P_QTORCAMENTO" => isset($row['QTORCAMENTO']) ? $row['QTORCAMENTO'] : 0,
            ":P_NRORG" => $this->environment->getNrOrg(),
            ":P_NRITORCAMESGOP" => $row['NRITORCAMESGOP']
        );
        try{
            $std = $this->connection->prepare($sqlUpdate);
            $std->execute($params);
        }catch(Exception $e) {
            throw $e;
        }
    }
 } 